// @flow

import React from 'react'
import { withReducer, compose } from 'recompose'
import { themr } from 'react-css-themr'
import Field from 'rebulma/lib/Field'
import Label from 'rebulma/lib/Label'
import Input from 'rebulma/lib/Input'
import Help from 'rebulma/lib/Help'
import Button from 'rebulma/lib/Button'

const defaultStyles = {
  label: '',
  input: '',
  button: '',
}

const ON_CHANGE = 'ON_CHANGE'

function validateField(name: string, value: string): string {
  switch (name) {
    case 'name':
      if (!value) return 'Required'
      break
    case 'email':
      if (!value) return 'Required'
      break
    case 'phone':
      if (!value) return 'Required'
      break
    case 'description':
      if (!value) return 'Required'
      break
    default:
      return ''
  }
  return ''
}

function reducer(state = { error: {} }, action) {
  const { name, value } = action.payload

  const error = {
    ...state.error,
    [name]: validateField(name, value),
  }

  const keys = Object.keys(error)
  const isValid = keys.map(key => error[key]).filter(x => !!x).length === 0

  switch (action.type) {
    case ON_CHANGE:
      return {
        ...state,
        [name]: value,
        error,
        isValid,
      }
    default:
      return state
  }
}

const ContactUs = ({
  className,
  nameLabel,
  emailLabel,
  phoneLabel,
  descriptionLabel,
  submitLabel,
  state: {
    name = '',
    email = '',
    phone = '',
    description = '',
    isValid = false,
    error,
  },
  dispatch,
  onSubmit,
  theme,
}: {
  className?: string,
  nameLabel?: string,
  emailLabel?: string,
  phoneLabel?: string,
  descriptionLabel?: string,
  submitLabel?: string,
  state: {
    name: string,
    email: string,
    phone: string,
    description: string,
    error: {
      name?: string,
      email?: string,
      phone?: string,
      description?: string,
    },
    isValid: boolean,
  },
  dispatch: Function,
  onSubmit?: Function,
  theme: { [key: string]: string },
}) => (
  <div className={className}>
    <div className="hero-body">
      <Field>
        <Label className={theme.label} htmlFor="name">{nameLabel}</Label>
        <Input
          className={theme.input}
          name="name"
          value={name}
          onChange={(e) => {
            dispatch({ type: ON_CHANGE, payload: { name: 'name', value: e.target.value } })
          }}
        />
        {error.name && (
          <Help className="is-danger">{error.name}</Help>
        )}
      </Field>
      <Field>
        <Label className={theme.label} htmlFor="email">{emailLabel}</Label>
        <Input
          className={theme.input}
          name="email"
          value={email}
          onChange={(e) => {
            dispatch({ type: ON_CHANGE, payload: { name: 'email', value: e.target.value } })
          }}
        />
        {error.email && (
          <Help className="is-danger">{error.email}</Help>
        )}
      </Field>
      <Field>
        <Label className={theme.label} htmlFor="phone">{phoneLabel}</Label>
        <Input
          className={theme.input}
          name="phone"
          value={phone}
          onChange={(e) => {
            dispatch({ type: ON_CHANGE, payload: { name: 'phone', value: e.target.value } })
          }}
        />
        {error.phone && (
          <Help className="is-danger">{error.phone}</Help>
        )}
      </Field>
      <Field>
        <Label className={theme.label} htmlFor="description">{descriptionLabel}</Label>
        <Input
          className={theme.input}
          name="description"
          value={description}
          onChange={(e) => {
            dispatch({ type: ON_CHANGE, payload: { name: 'description', value: e.target.value } })
          }}
        />
        {error.description && (
          <Help className="is-danger">{error.description}</Help>
        )}
      </Field>
      <Field>
        <Button
          control
          disabled={!isValid}
          className={theme.button}
          onClick={() => {
            if (onSubmit) onSubmit({ name, email, phone, description })
          }}
        >
          {submitLabel}
        </Button>
      </Field>
    </div>
  </div>
)

ContactUs.defaultProps = {
  children: null,
  className: null,
  nameLabel: 'Name',
  emailLabel: 'Email',
  phoneLabel: 'Phone',
  descriptionLabel: 'Description',
  submitLabel: 'Submit',
  error: {},
  onSubmit: null,
}

const defaultError = {
  name: 'Required',
  email: 'Required',
  phone: 'Required',
  description: 'Required',
}

export default compose(
  themr('ContactUs', defaultStyles, { composeTheme: 'softly' }),
  withReducer('state', 'dispatch', reducer, { error: defaultError, isValid: false }),
)(ContactUs)
